from difflib import SequenceMatcher
from .entry import KarteiEntry
from pathlib import Path
import argparse
import random
import secrets
import sys


def main():
    celebration_emojis = [
        "🎈",
        "🎉",
        "🎊",
        "🎓",
        "🥳",
        "🍾",
    ]

    parser = argparse.ArgumentParser(description="Kartei Vocabulary Trainer")
    parser.add_argument("file", type=Path, help="path to file")
    args = parser.parse_args()

    try:
        content = args.file.read_text()
    except FileNotFoundError as e:
        print(e)
        sys.exit(1)

    entries = []
    for line in content.splitlines():
        if line.strip() == "":
            pass
        else:
            entries.append(KarteiEntry(line))

    random.shuffle(entries)

    for entry in entries:
        print("   " + entry.left)
        try:
            text = input(">> ").strip()
        except KeyboardInterrupt:
            print()
            sys.exit(0)
        similarity = SequenceMatcher(None, entry.right, text).ratio()
        if similarity == 1.0:
            print("\x1b[1;32m   " + entry.right, f"(similarity: {similarity:.2f} {secrets.choice(celebration_emojis)})\x1b[0m")
        elif similarity >= 0.86:
            print("\x1b[1;33m   " + entry.right, f"(similarity: {similarity:.2f})\x1b[0m")
        else:
            print("\x1b[1;31m   " + entry.right, f"(similarity: {similarity:.2f})\x1b[0m")
        print()

if __name__ == "__main__":
    main()



from typing import Dict, Optional


class KarteiEntry(object):
    entry_sep = "=="
    param_sep = "#"
    p_sep = "="

    def __init__(self, raw_text: str) -> None:
        self._raw_text: str = raw_text.strip()
        self._left_side: str
        self._right_side: str
        self._params: Optional[Dict[str, str]] = None
        self._load_entry()
        self._load_entry_params()

    def _load_entry(self) -> None:
        entry: str = self._raw_text.partition(self.param_sep)[0].strip()
        self._left_side = entry.partition(self.entry_sep)[0].strip()
        self._right_side = entry.partition(self.entry_sep)[2].strip()

    def _load_entry_params(self) -> None:
        entry_params: str = self._raw_text.partition(self.param_sep)[2]
        try:
            self._params = {
                param.split(self.p_sep)[0].strip(): param.split(self.p_sep)[1].strip()
                for param in entry_params.split()
            }
        except IndexError:
            pass

    @property
    def left(self) -> str:
        return self._left_side.strip()

    @property
    def right(self) -> str:
        return self._right_side.strip()

    @property
    def difficulty(self) -> int:
        if self._params is not None:
            result: Optional[str] = self._params.get("diff")
            if result is not None:
                try:
                    return int(result)
                except ValueError:
                    return 0
            else:
                return 0
        else:
            return 0

    def __repr__(self) -> str:
        return str(self)

    def __str__(self) -> str:
        return f"KarteiEntry(left=\"{self.left}\", right=\"{self.right}\", difficulty={self.difficulty})"
